# SPDX-FileCopyrightText : 2023 Detlef Gebhardt, written for CircuitPython
# SPDX-FileCopyrightText : Copyright (c) 2023 Detlef Gebhardt
# SPDX-Filename          : Pico-Glücksrad  (20.12.2023)
# SPDX-License-Identifier: GEBMEDIA
import time
import gc
import board
import math
import random
import busio
import displayio
import digitalio
import terminalio
import bitmaptools
from adafruit_display_shapes.circle import Circle
from adafruit_display_text import label
import gc9a01
import my_qmi8658
import adafruit_imageload
from adafruit_ticks import ticks_ms

# Release any resources currently in use for the displays
displayio.release_displays()

# Make the displayio SPI bus and the GC9A01 display
spi = busio.SPI(clock=board.LCD_CLK, MOSI=board.LCD_DIN)
display_bus = displayio.FourWire(spi, command=board.LCD_DC, chip_select=board.LCD_CS, reset=board.LCD_RST)
display = gc9a01.GC9A01(display_bus, width=240, height=240, backlight_pin=board.LCD_BL)
display.rotation = 0
display.brightness = 1
main = displayio.Group()
display.show(main)

# Sensor initialisieren
sensor=my_qmi8658.QMI8658()

# Variable definieren
xpos_rot = 116
ypos_rot = 70
xpos=120
ypos=120
width = 240
height = 240
step = 100
count = 0
point = 0
alpha_rad = 0

# Koordinaten fuer den Login der Kugel
# (112,20),(154,34),(188,70),(200,112),(186,154),(157,185),(112,200),(67,189),(36,160),(22,115),(32,70),(67,34)
feld_x = [112,154,188,200,186,157,112,67,36,22,32,67,]
feld_y = [20,34,70,112,154,187,200,189,160,115,70,34]
# Punkte auf dem Farbkreis
points = [100,25,50,10,100,25,50,10,100,25,50,10]

sort1 = [100,25,50,10,100,25,50,10,100,25,50,10]
sort2 = [25,50,10,100,25,50,10,100,25,50,10,100]
sort3 = [50,10,100,25,50,10,100,25,50,10,100,25]
sort4 = [10,100,25,50,10,100,25,50,10,100,25,50]

# Bitmap-Dateien für Hintergrund und Zeiger
zifferblatt = "/gl_images/zifferblatt.bmp"
ring_1 = "/gl_images/farbring_1.bmp"

## 240x240 Hintergrundbild mit Farbkreis
bg_bitmap, bg_pal = adafruit_imageload.load(zifferblatt, bitmap=displayio.Bitmap,palette=displayio.Palette)
bg_pal.make_transparent(0)
bg_bitmap_scribble = displayio.Bitmap(display.width, display.height, len(bg_pal))
bg_tile_grid = displayio.TileGrid(bg_bitmap_scribble, pixel_shader=bg_pal)
main.append(bg_tile_grid)
bitmaptools.rotozoom( bg_bitmap_scribble, bg_bitmap, angle = 0)

## rotierender Farbring
ring1_bitmap, ring1_palette = adafruit_imageload.load(ring_1, bitmap=displayio.Bitmap,palette=displayio.Palette)
ring1_palette.make_transparent(0)
## Blank bitmap fuer rotierenden Farbring
ring1_bitmap_blank = displayio.Bitmap(ring1_bitmap.width, ring1_bitmap.height, 1)
ring1_bitmap_scribble = displayio.Bitmap(display.width, display.height, len(ring1_palette))
ring1_tile_grid = displayio.TileGrid(ring1_bitmap_scribble, pixel_shader=ring1_palette, x=0,y=0)
main.append(ring1_tile_grid)

# Label fuer Punkteanzeige
updating_points_2 = label.Label(font=terminalio.FONT, text="000" + str(point), scale=3, color=0xffff00, line_spacing=1)
updating_points_2.anchor_point = (0, 0)
updating_points_2.anchored_position = (85, 155)
main.append(updating_points_2)

## Def. Loginloch und umlaufende Kugel
circle_hole = Circle(116, 65, 10, fill=0x1a1a1a, outline=0x000000)
main.append(circle_hole)
circle_rot = Circle(xpos_rot, ypos_rot, 6, fill=0xffff00, outline=0x000000)
main.append(circle_rot)

# Kugel zum Einloggen
circle_log = Circle(xpos, ypos, 8, fill=0xff0000, outline=None)
main.append(circle_log)

def login(stop, start, xpos, ypos, point, punkt):
    global newpoint
    newpoint=0
    while(True):
        # Sensor bei jedem Durchlauf auslesen und auswerten
        xyz=sensor.Read_XYZ()
        # Display wird rel. zur x-Achse bewegt
        wert_x = (10)*xyz[0]
        # Display wird rel. zur y-Achse bewegt
        wert_y = (10)*xyz[1]
        # Bewegung hoch
        if wert_x < - 0.5 and wert_y > - 0.5:
            ypos = ypos + 1
            if ypos < 10:
                ypos = 10
        circle_log.y = ypos
        # Bewegung runter
        if wert_x > 0.5 and wert_y < 0.5:
            ypos = ypos - 1
            if ypos > 230:
                ypos = 230
        circle_log.y = ypos
        # Bewegung rechts/links
        if wert_x > - 0.5 and wert_y < - 0.5:
            xpos = xpos -1
            if xpos < 10:
                xpos = 10
        circle_log.x =xpos
        if wert_x > - 0.5 and wert_y > 0.5:
            xpos = xpos + 1
            if xpos > 230:
                xpos = 230
        circle_log.x =xpos
        # Zeit ist abgelaufen (nicht eingeloggt)
        if (ticks_ms() - start)/1000 > 10:
            xpos = 112
            ypos = 112
            circle_log.x =xpos
            circle_log.y =ypos
            return newpoint  #wird mit Null zurueckgegeben
        # Kugel wurde eingeloggt
        if (abs(xpos-feld_x[stop])<6 and abs(ypos-feld_y[stop])<6) :
            circle_log.x = feld_x[stop]
            circle_log.y = feld_y[stop]
            time.sleep(1)
            newpoint = point + punkt
            xpos = 112
            ypos = 112
            circle_log.x =xpos
            circle_log.y =ypos
            return newpoint  #neuer Punktewert wird zurueckgegeben 
 
start = ticks_ms()
zahl = random.randint(20, 50)

while True:  
    ## Schrittezahl erhoehen
    if (ticks_ms() - start)/1000 > 0.05:
        count += 1
        start = ticks_ms()  
    ## Punktezaehler
    if (step - zahl) <= 0:
        # position der umlaufenden Kugel
        stop = (count - 60*int((100-zahl)/60))//5  
        punkt = points[stop]
        circle_rot.fill = 0x000000
        circle_hole.fill = 0xffff00
        circle_hole.x = feld_x[stop]
        circle_hole.y = feld_y[stop]
        # Kugel einloggen
        start = ticks_ms()
        login(stop, start, xpos, ypos, point, punkt)
        # Punktestand aktualisieren
        if newpoint != 0:
            point = newpoint
        if point == 0:
            updating_points_2.text = "000" + str(point)
        if point >= 10 and point< 100:
            updating_points_2.text = "00" + str(point)
        if point >= 100 and point < 1000:
            updating_points_2.text = "0" + str(point)
        if point >= 1000:
            updating_points_2.text = str(point)
        if point >= 10000:
            point = 0
            updating_points_2.text = "000" + str(point)
        circle_rot.fill = 0xffff00
        circle_hole.fill = 0x1a1a1a
        circle_hole.x = 116
        circle_hole.y = 65     
        # gefallene Punktzahl und Hintergrund nach unten drehen
        step = 100
        zahl = random.randint(20, 50)
        count = 0
        start = ticks_ms()
        alpha_rad = alpha_rad + (stop+6)*math.pi/(-6)
        bitmaptools.rotozoom( bg_bitmap_scribble, bg_bitmap, angle = alpha_rad)
        # Punkte entsp. der Drehung umsortieren
        if punkt == 100:
            points = sort3
        if punkt == 25:
            points = sort4
        if punkt == 50:
            points = sort1
        if punkt == 10:
            points = sort2        
        circle_rot.x = 116
        xpos_rot = 116
        circle_rot.y = 70
        ypos_rot = 70
        
    ## rotierendes Farbring im Zentrum
    step -= 1
    alpha_rad_step = step
    bitmaptools.rotozoom( ring1_bitmap_scribble, ring1_bitmap, angle = alpha_rad_step, px=32, py=32)
        
    ## umlaufende Kugel      
    xpos_neu = int(width/2 + 50*math.cos((count -15)*math.pi/30))
    delta_x = xpos_neu - xpos_rot
    xpos_rot = xpos_neu
    ypos_neu = int(height/2 + 50*math.sin((count -15)*math.pi/30))
    delta_y = ypos_neu - ypos_rot
    ypos_rot = ypos_neu
    circle_rot.x = circle_rot.x + delta_x
    circle_rot.y = circle_rot.y + delta_y
     
    gc.collect()   
    #print(gc.mem_free())